@@ -8,6 +8,7 @@
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007, 2008 Chris Moore
 **	Copyright (c) 2012-2013 Carlos López
+**	......... ... 2014-2017 Ivan Mahonin
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -51,13 +52,16 @@ namespace synfig {
 */
 class Layer_PasteCanvas : public Layer_Composite, public Layer_NoDeform
 {
+public:
+	typedef etl::handle<Layer_PasteCanvas> Handle;
+
 private:
 	//! Parameter: (Origin) Position offset
 	ValueBase param_origin;
-	//! Parameter: (Transfromation) Position, rotation and scale of the paste canvas layer
+	//! Parameter: (Transformation) Position, rotation and scale of the paste canvas layer
 	ValueBase param_transformation;
 	//! Parameter: (etl::loose_handle<synfig::Canvas>) The canvas parameter
-	etl::loose_handle<synfig::Canvas> canvas;
+	etl::loose_handle<synfig::Canvas> sub_canvas;
 	//! Parameter: (Real) Time dilation of the paste canvas layer
 	ValueBase param_time_dilation;
 	//! Parameter: (Time) Time offset of the paste canvas layer
@@ -134,7 +138,7 @@ public:
 
 	//! Gets the canvas parameter. It is called sub_canvas to avoid confusion
 	//! with the get_canvas from the Layer class.
-	etl::handle<synfig::Canvas> get_sub_canvas()const { return canvas; }
+	etl::handle<synfig::Canvas> get_sub_canvas()const { return sub_canvas; }
 	//! Sets the canvas parameter.
 	//! \see get_sub_canvas()
 	void set_sub_canvas(etl::handle<synfig::Canvas> x);
@@ -154,7 +158,7 @@ public:
 	}
 
 	//! Default constructor
-	Layer_PasteCanvas();
+	explicit Layer_PasteCanvas(Real amount = 1.0, Color::BlendMethod blend_method = Color::BLEND_COMPOSITE);
 	//! Destructor
 	virtual ~Layer_PasteCanvas();
 	//! Returns a string with the localized name of this layer
@@ -168,10 +172,6 @@ public:
 	//! Gets the blend color of the Layer in the context at \a pos
 	virtual Color get_color(Context context, const Point &pos)const;
 
-	//! Renders the Canvas to the given Surface in an accelerated manner
-	//! See Layer::accelerated_render
-	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 	//! Bounding rect for this layer depends from context_params
 	Rect get_bounding_rect_context_dependent(const ContextParams &context_params)const;
 	//!Returns the rectangle that includes the context of the layer and
@@ -181,15 +181,18 @@ public:
 	virtual Vocab get_param_vocab()const;
 	//! Checks to see if a part of the Paste Canvas Layer is directly under \a point
 	virtual synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
-	virtual void set_render_method(Context context, RenderMethod x);
 
 	virtual void fill_sound_processor(SoundProcessor &soundProcessor) const;
 
 	virtual void on_childs_changed() { }
 
 protected:
+	virtual Context build_context_queue(Context context, CanvasBase &out_queue)const;
+
 	//! Sets the time of the Paste Canvas Layer and those under it
 	virtual void set_time_vfunc(IndependentContext context, Time time)const;
+	//! Loads external resources (frames) for child layers of the Paste Canvas Layer
+	virtual void load_resources_vfunc(IndependentContext context, Time time)const;
 	//! Sets the outline_grow of the Paste Canvas Layer and those under it
 	virtual void set_outline_grow_vfunc(IndependentContext context, Real outline_grow)const;
 	//!	Function to be overloaded that fills the Time Point Set with
@@ -198,7 +201,7 @@ protected:
 	//! Layer time points. \todo clarify all this comments.
 	virtual void get_times_vfunc(Node::time_set &set) const;
 
-	virtual rendering::Task::Handle build_composite_task_vfunc(ContextParams context_params)const;
+	virtual rendering::Task::Handle build_rendering_task_vfunc(Context context)const;
 }; // END of class Layer_PasteCanvas
 
 }; // END of namespace synfig
